@@ -26,6 +26,7 @@ import shlex
 import signal
 import sys
 import time
+import warnings
 
 import enum
 from oslo_utils import encodeutils
@@ -36,6 +37,11 @@ from oslo_utils import timeutils
 from oslo_concurrency._i18n import _
 
 
+if os.name == 'nt':
+    warnings.warn('Support for Windows OS is deprecated.',
+                  category=DeprecationWarning, stacklevel=3)
+
+
 # NOTE(bnemec): eventlet doesn't monkey patch subprocess, so we need to
 # determine the proper subprocess module to use ourselves.  I'm using the
 # time module as the check because that's a monkey patched module we use
@@ -55,6 +61,13 @@ if eventlet_patched:
         from eventlet.green import subprocess
 
     from eventlet import tpool
+
+    # Monkey patch the original current_thread to use the up-to-date _active
+    # global variable. See https://bugs.launchpad.net/bugs/1863021 and
+    # https://github.com/eventlet/eventlet/issues/592
+    import __original_module_threading as orig_threading
+    import threading
+    orig_threading.current_thread.__globals__['_active'] = threading._active
 else:
     import subprocess
 
@@ -262,8 +275,8 @@ def execute(*cmd, **kwargs):
                             below for a detailed description.
     :type prlimit:          :class:`ProcessLimits`
     :param python_exec:     The python executable to use for enforcing
-                            prlimits. If this is not set it will default to use
-                            sys.executable.
+                            prlimits. If this is not set or is None, it will
+                            default to use sys.executable.
     :type python_exec:      string
     :param timeout:         Timeout (in seconds) to wait for the process
                             termination. If timeout is reached,
@@ -322,7 +335,7 @@ def execute(*cmd, **kwargs):
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
     prlimit = kwargs.pop('prlimit', None)
-    python_exec = kwargs.pop('python_exec', sys.executable)
+    python_exec = kwargs.pop('python_exec', None) or sys.executable
     timeout = kwargs.pop('timeout', None)
 
     if isinstance(check_exit_code, bool):
