@@ -13,6 +13,7 @@
 #    under the License.
 
 import collections
+import errno
 import multiprocessing
 import os
 import signal
@@ -185,6 +186,13 @@ class LockTestCase(test_base.BaseTestCase):
                 pass
         self.assertRaises(NotImplementedError, do_test)
 
+    def test_fair_lock_with_nonblocking(self):
+        def do_test():
+            with lockutils.lock('testlock', 'test-', fair=True,
+                                blocking=False):
+                pass
+        self.assertRaises(NotImplementedError, do_test)
+
     def test_nested_synchronized_external_works(self):
         """We can nest external syncs."""
         self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
@@ -320,8 +328,8 @@ class LockTestCase(test_base.BaseTestCase):
         remove_mock.assert_called_once_with(path_mock.return_value)
         log_mock.assert_not_called()
 
-    @mock.patch('logging.Logger.info')
-    @mock.patch('os.remove', side_effect=OSError)
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.ENOENT, None))
     @mock.patch('oslo_concurrency.lockutils._get_lock_path')
     def test_remove_lock_external_file_doesnt_exists(self, path_mock,
                                                      remove_mock, log_mock):
@@ -332,6 +340,20 @@ class LockTestCase(test_base.BaseTestCase):
                                           mock.sentinel.prefix,
                                           mock.sentinel.lock_path)
         remove_mock.assert_called_once_with(path_mock.return_value)
+        log_mock.assert_not_called()
+
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.EPERM, None))
+    @mock.patch('oslo_concurrency.lockutils._get_lock_path')
+    def test_remove_lock_external_file_permission_error(
+            self, path_mock, remove_mock, log_mock):
+        lockutils.remove_external_lock_file(mock.sentinel.name,
+                                            mock.sentinel.prefix,
+                                            mock.sentinel.lock_path)
+        path_mock.assert_called_once_with(mock.sentinel.name,
+                                          mock.sentinel.prefix,
+                                          mock.sentinel.lock_path)
+        remove_mock.assert_called_once_with(path_mock.return_value)
         log_mock.assert_called()
 
     def test_no_slash_in_b64(self):
@@ -415,6 +437,35 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
                 time.sleep(.5)
                 os._exit(0)
 
+    def test_interprocess_nonblocking_external_lock(self):
+        """Check that we're not actually blocking between processes."""
+
+        nb_calls = multiprocessing.Value('i', 0)
+
+        @lockutils.synchronized('foo', blocking=False, external=True,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running operation in a process."""
+            param.value += 1
+            time.sleep(.5)
+
+        def other(param):
+            foo(param)
+
+        process = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process.start()
+        # Make sure the other process grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for process to grab lock')
+            time.sleep(0)
+        process1 = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process1.start()
+        process1.join()
+        process.join()
+        self.assertEqual(1, nb_calls.value)
+
     def test_interthread_external_lock(self):
         call_list = []
 
@@ -450,6 +501,62 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
         thread.join()
         self.assertEqual(['other', 'other', 'main', 'main'], call_list)
 
+    def test_interthread_nonblocking_external_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', external=True, blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running threaded operation."""
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
+    def test_interthread_nonblocking_internal_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            # Simulate a long-running threaded operation.
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not call_list:
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
     def test_non_destructive(self):
         lock_file = os.path.join(self.lock_dir, 'not-destroyed')
         with open(lock_file, 'w') as f:
